###############################################################################
## Vote System 1.1
## KobazVote.tcl
## First Public Release
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazTools.tcl
##           KobazConfig.tcl
##
## FEATURES: A vote tracking system.
## 
## COMMANDS: Arguments in "< >" are mandatory.
##           Arguments in "? ?" are optional.  
##           Arguments in "[ ]" are a toggle, and mandatory.
##           Arguments in "( )" must be used together
## 
## --Via dcc:
## .vote                    -  Set up a vote
## --Via msg
## !vote <choice>           -  Vote for choice number <choice>
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################


source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazVote {

####
# Load required libraries

kbrequire Tools
kbrequire Config

# end
####

namespace import -force ::KobazTools::*
namespace import -force ::KobazConfig::*

proc init {} {

    bind dcc m vote  KobazVote::dcc_vote
    bind msg - !vote KobazVote::msg_vote

    putlog "KobazVote.tcl by Kobaz Loaded."
}

proc dcc_vote {hand idx arg} {
    global nick
    set flag [lindex $arg 0]
    set text [lrange $arg 1 end]
    set channel [string tolower $text]

    if {$flag == "-k"} {
        if {$channel == ""} {
            putdcc $idx "You must specify a channel"
            return
        }
	if {$channel != [get_config KobazVote channel]} {
	    putdcc $idx "Vote not in progess on $text"
	    return
	} else {
	    putdcc $idx "Killed vote on $text"
	    set_config KobazVote channel ""
	    set timer [utimerexists "KobazVote::vote_done $text"]
	    killutimer $timer
	    tellnick pub $text "Vote killed"
	    return
	}
    }

    if {[get_config KobazVote channel] != ""} {
	putdcc $idx "Vote in progress on [get_config KobazVote channel]"

	putdcc $idx "Vote config:"
	putdcc $idx "Set question: .vote -q <question>"
	putdcc $idx "Set answers:  .vote -a <answer>"
	putdcc $idx "Set time:     .vote -t <minutes>"
	putdcc $idx "Kill vote:    .vote -k <channel>"
	putdcc $idx ""
	putdcc $idx "When finished: .vote -d <channel>"
	putdcc $idx "To reset .vote -reset"
	putdcc $idx "To list current config: .vote -l"
	
	return
    }

    switch -- $flag {
	-q {
	    set_config KobazVote question $text
	    putdcc $idx "Vote: Ok."
	    return
	}
	-a {
	    set answers [get_config KobazVote answers]
	    lappend answers $text
	    set_config KobazVote answers $answers
	    putdcc $idx "Vote: Ok."
	    return
	}
	-t {
	    set_config KobazVote time $text
	    putdcc $idx "Vote: Ok."
	    return
	}
	-d {
	    if {$text == ""} {
		putdcc $idx "Vote: Please specify a channel"
		return
	    }

	    if {[lsearch [string tolower [channels]] $text] == "-1"} {
		putdcc $idx "Vote: I'm not on channel ($text)"
		return
	    }

#	    bind sign - "$channel *"    "KobazVote::remove_vote_quit"
#	    bind part - "$channel *"    "KobazVote::remove_vote_quit"
#	    bind kick - "$channel *"    "KobazVote::remove_vote_kick"
	    
	    putdcc $idx "Vote: Displaying vote in: $text."
	    set_config KobazVote channel $text

	    tellnick pub $text "Vote: [get_config KobazVote question]"
	    tellnick pub $text "To place your vote type: /msg $nick !vote $text <number of choice>"

	    set anum 0
	    foreach answer [get_config KobazVote answers] {
		incr anum
		tellnick pub $text "Choice($anum): $answer"
		set_config KobazVote choice$anum "0"
	    }

	    set time [get_config KobazVote time]
	    if {$time == "1"} {set s ""} else {set s "s"}
	    set timer [expr $time * 60]
	    
	    tellnick pub $text "Vote will end in [get_config KobazVote time] minute$s."
	    utimer $timer "KobazVote::vote_done $text"
	    return
	}
	-l {
	    putdcc $idx "Question: [get_config KobazVote question]"
	    set anum 0
	    foreach answer [get_config KobazVote answers] {
		incr anum
		putdcc $idx "Choice($anum): $answer"
	    }
	    putdcc $idx "Time: [get_config KobazVote time] minutes"
	    return
	}
	-reset {
	    foreach config {question answers time voted} {
		set_config KobazVote $config ""
	    }
	    putdcc $idx "Vote: Ok."
	    return
	}
    }

    putdcc $idx "Vote config:"
    putdcc $idx "Set question: .vote -q <question>"
    putdcc $idx "Set answers:  .vote -a <answer>"
    putdcc $idx "Set time:     .vote -t <minutes>"
    putdcc $idx "Kill vote:    .vote -k <channel>"
    putdcc $idx ""
    putdcc $idx "When finished: .vote -d <channel>"
    putdcc $idx "To reset .vote -reset"
    putdcc $idx "To list current config: .vote -l"
    
    return
}

proc msg_vote {nick uhost handle arg} {
    set channel [lindex $arg 0]
    set arg [lindex $arg 1]
    if {[get_config KobazVote channel] != $channel} {
	tellnick notice $nick "Vote currently not in progess."
	return
    }

    regsub -all {[0-9]} $arg "" nonum

    if {$nonum != ""} {
	tellnick notice $nick "Vote: Argument must be a number."
	return
    }

    if {[lsearch [get_config KobazVote voted] $uhost] != "-1"} {
	tellnick notice $nick "Vote: You have already voted."
	return
    }

    if {$arg > [llength [get_config KobazVote answers]] || $arg < 1} {
	tellnick notice $nick "Vote: There is no choice(${arg})."
	return
    }

    set voted [get_config KobazVote voted]
    lappend voted $uhost
    set_config KobazVote voted $voted

    set choice [get_config KobazVote choice$arg]
    incr choice
    set_config KobazVote choice$arg $choice

    tellnick notice $nick "Vote: Your vote for choice($arg) has been recorded."

    return
}

proc remove_vote_quit {} {
    set choice [get_config KobazVote choice
   
    return
}


proc vote_done {channel} {
    set_config KobazVote channel ""

    tellnick pub $channel "Vote has ended. Final tallys"
    
    set anum 0
    set answers ""
    foreach answer [get_config KobazVote answers] {
	incr anum
	lappend answers "$anum [get_config KobazVote choice$anum]"
    }    

    set choicelist [lsort -dictionary -decreasing -index 1 $answers]
    foreach answer $choicelist {
	set answer [lindex $answer 0]
	set a [lindex [get_config KobazVote answers] [expr $answer -1]]
	tellnick pub $channel "($answer)$a: [get_config KobazVote choice$answer]"
    }

    foreach config {voted} {
	set_config KobazVote $config ""
    }
    return
}

init
}



